#!/bin/bash
#
# Copyright (c) 2020 All Rights Reserved iLimulus Computing
VERSIONDATE="Sun Dec 27 11:29:34 EST 2020"
# The script sets up a DCHP kickstart install for the HDP worker nodes. Most of the 
# script assumes things work and bases actions on the simple responses (ping
# or ssh hostanme)
# Also assumes configuration and template files are installed previously.
# Notes:
#  1. Sometimes the node build gets stuck with the interface not connecting
#  to the server. Retrying seems to work (which is why the reimage specific nodes is needed)
#  2. With no extra NICs, ETH should be "eth0" (WANETH needs to be changed if not the build eth)
# 
#
# TODO: 1. Allow for individual nodes to be reimaged, list possible nodes, select one node.
#       2. muliple machine configuration
#          For each SUBMASTER
#             - ask to poweron submaster (needs to have DHCP running, when build, use /data instead of home)
#             - build SUBMASTER, then check to make sure "ssh SUBMASTER ls /var/run/relayset"
#             - if not there, exit
#             - if there then loop through config, use ssh SUBMASTER relayset 
#             - names SUBMASTER-n0, SUBMASTER-n1 etc place in /etc/hosts.
#             - modify node-poweron/off to recognize SUBMASTER-nX, if SUBMASTER, then 
#             - first check if SUBMASTER is up, if not, WARNING not not possible because 
#             - if up, then turn node off or on
#        3. For 8 mothboard systems with 10 GbE, n3 needs different Ethernet config, usuall eth4
# NODES must be listed in a very specific fashion
#    node-number:[host]-subhost 

# node-number = the actual node number listed in /etc/hosts
# [host] = if this system is a sub-host
# sub-host  is the name of the host that controls the nodes power
# the list must be in order, for limulus with 3 nodes, limulus2 is
# is the name of the sub-host (nodes 3,4,5)
#    3:host-limulus2 4:limulus2 5:limulus2 6:limulus2 
# There is no specific order of sub-hosts as long as the nodes
# controled by the sub-host are in the above format (and order)

# >>>>>> CONFIGURATION NEEDED <<<<<<<<
NAMESERVER=192.168.93.1
ISOSOURCE="https://www.limulus-computing.com/download/releases/co/7/limulus/x86_64/extra"
HTTPISOPATH="/var/www/html/iso"
ISOPATH="/var/tmp"
ISO="CentOS-7-x86_64-Minimal-2009.iso"
KSPATH="/var/www/html/ks"
KSTEMPLATEPATH="/etc/warewulf/.Limulus/post-install/hadoop"
KSNAME="ks.cfg.hdp-3.1.4-nodes"
ETH="eth0"
DEFALTPATH="/var/lib/tftpboot/pxelinux.cfg"
DEFAULTNAME="default"
IPBASE=10.0.0
IPSTART=10
RELAYSTART=2
# REMOVE # Configure customer ($SERIAL)specifc things.
# REMOVE # read it the /etc/hosts file
# REMOVE while read line ; do
# REMOVE   HOSTS="$HOSTS$line\n"
# REMOVE done < $SERIAL/ADD_TO_HOSTS_FILE
# REMOVE  read in the node build config
# REMOVE NODES=`cat $SERIAL/NODE_CONFIG`

# >>>>>> END CONFIGURATION NEEDED <<<<<<<<

# read it the /etc/hosts file
$HOSTS=""
while read line ; do
  HOSTS="$HOSTS$line\n"
done < ./HOSTS-FILE
NODES=$(cat /etc/warewulf/.Limulus/actual-nodes)
MAPNODE=/etc/warewulf/.Limulus/limulus-map-node

SLEEPWAIT=30
SLEEPSHORT=5 
# Set this accordingly, need to make this more automatic and based on external hosts file
# also, set local /etc/pingd.conf to only local nodes (used for node-poweron/off)
COUNT=0
MAXCOUNT=60
DONE="1"
GATEWAY=10.0.0.1
# On 10 GbE systems, the interface number is different (VERIFY)
# login nodes: the ethernet build network 10.0.0. is  eth4
# worker nodes, eth3 or 2 
# dined for specific multi-system build
HOSTKEY=$(cat /root/.ssh/id_ecdsa.pub)
printf "\nLimulus Hadoop $VERSIONDATE
Copyright (c) 2020 All Rights Reserved Limulus Computing 
\nThis program will automatically register ALL the Amabri worker nodes
on this host. It needs to be run successfully one time and there is no
need to run it again if the nodes are booting and operating correctly.
THIS PROGRAM WILL DESTROY ANY DATA ON THE WORKER NODES!
Enter 'y' to continue, all other keys quit: "
read ANS
if test "$ANS" != "y" ; then
  exit
fi
echo ""
# Make sure services running and ISO is mounted
echo "Set up files and needed services"
echo "  Checking for CentOS Minimal ISO: "
if [ -f $ISOPATH/$ISO ]; then
  echo "  Found $ISOPATH/$ISO"
else
  echo -n "  ISO not found, downloading from $ISOSOURCE: "
  wget --no-check-certificate -P $ISOPATH $ISOSOURCE/$ISO
  if [ "$?" -ne 0 ]; then
    echo "FAILED Exiting"
    exit 1
  fi
  echo "SUCCESS"
fi
/bin/mkdir -p $HTTPISOPATH
echo -n "  Mounting ISO under $HTTPISOPATH: "
umount $HTTPISOPATH 1> /dev/null
mount -o loop $ISOPATH/$ISO $HTTPISOPATH 1> /dev/null
if [ "$?" -ne 0 ]; then
  echo "FAILED Exiting"
  exit 1
fi
echo "SUCCESS"
systemctl restart httpd  &> /dev/null
if [ "$?" -ne 0 ]; then
  echo "FAILED Exiting"
  exit 1
fi 
echo -n "  Starting webserver (httpd): "
systemctl restart httpd  &> /dev/null
if [ "$?" -ne 0 ]; then
  echo "FAILED Exiting"
  exit 1
fi 
echo "SUCCESS"
echo -n "  Starting netboot server (dhcpd): "
systemctl restart dhcpd &> /dev/null
if [ "$?" -ne 0 ]; then
  echo "FAILED Exiting"
  exit 1
fi 
echo "SUCCESS"

echo "Begin Node Install. Progress is reported every $SLEEPWAIT seconds."
# make kickstart path
/bin/mkdir -p $KSPATH

# reset and turn everything off
relayset init  &> /dev/null
relayset 2 off; relayset 3 off; relayset 4 off
sleep $SLEEPSHORT
# Start registration loop
# Note: for systems that are installed and boot from the M.2 drive as a first priority
# you will need to force a PXE boot (BIOS is set as M.2 first, then PXE)
# The MBR can be removed on the nodes by running (ON THE NODES, ASSUMES sda,
# but could be nvme0n1 or similar) LOG INTO NODE!
#     dd if=/dev/zero of=/dev/sda bs=512 count=1
# Or, attache a monitor and keyboard on the node during this process (not
# possible with Ryzen based nodes) and select BIOS/ boot menu, then choose 
# the IBA (Intel boot agent)

for N in $NODES; do
  NODE_KSNAME="$N"_$KSNAME
  echo "-----------------------"
  echo "Registering $N"
  echo "  Creating the default file "
  sed  "s|_FILL_IN_ETH_|$ETH|g" $DEFALTPATH/$DEFAULTNAME.template >$DEFALTPATH/$DEFAULTNAME
  sed  --in-place "s|_FILL_IN_KS_FILE_|$NODE_KSNAME|g" $DEFALTPATH/$DEFAULTNAME 
  echo "  Creating the $N kickstart file "
  sed  "s|_FILL_IN_ETH_|$ETH|g" $KSTEMPLATEPATH/$KSNAME.template >  $KSPATH/$NODE_KSNAME
  sed --in-place  "s|_FILL_IN_WANETH_|$ETH|g" $KSPATH/$NODE_KSNAME 
  sed --in-place  "s|_FILL_IN_HOSTNAME_|$N|g" $KSPATH/$NODE_KSNAME
  sed --in-place  "s|_FILL_IN_NS_|$NAMESERVER|g" $KSPATH/$NODE_KSNAME
  sed --in-place  "s|_FILL_IN_IP_|$IPBASE.$IPSTART|g" $KSPATH/$NODE_KSNAME
  sed --in-place  "s|_FILL_IN_HOSTKEY_|$HOSTKEY|g" $KSPATH/$NODE_KSNAME
  sed --in-place  "s|__ADD_TO_HOSTS_FILE__|$HOSTS|g" $KSPATH/$NODE_KSNAME
  echo "  Using files $DEFALTPATH/$DEFAULTNAME $KSPATH/$NODE_KSNAME"
  
# turn node on
  COUNT=0
  BUILDCOUNT=0
  DONE="no"
  # build loop for each node, turn on the node
  relayset $RELAYSTART on

# At this point the node is assumed to be powered on and in DHCP mode 
  while [ "$DONE" != "yes" ]; do  
    # wait for node to start, if it is requesting DHCP, the tftp boot
    # process takes over, and we wait for ping response, if immediate,
    # then node probably was previously installed.  
    #
    sleep $SLEEPWAIT
    /bin/ping -q -c 1 $N >& /dev/null
    if [ "$?" -ne 0 ]; then
       if [ $BUILDCOUNT -gt 10 ] ; then
         echo "  $N is Rebooting. Count=$COUNT"
       else  
         echo "  $N is NOT responding. Count=$COUNT"
       fi
    else
      NAME=$(/usr/bin/ssh $N hostname -a)
      if [ "$NAME" == "" ]; then
        NAME=$(/usr/bin/ssh $N hostname)
      fi
      if [ "$?" -ne 0 ]; then
        echo "  $N is responding: building. Count=$COUNT"
        let BUILDCOUNT=$BUILDCOUNT+1
      else
        if [ "$NAME" == "$N" ]; then 
          if [ $BUILDCOUNT -lt 2 ]; then
            echo "!! Node $N seems to have been previously installed."
            echo "Do you want to re-install $N?  (ALL DATA WILL BE LOST)"
            echo -n 'Enter "y" to re-install systems, any other key to skip: '
            read ANS
            if test "$ANS" = "y" ; then
               echo "The MBR on /dev/sda will be deleted. This action should"
               echo "trigger a network (PXE) boot and new install. If $N does not"
               echo "PXE boot or seems to get stuck, attach a monitor/kbd to node $N "
               echo "and cycle the power using the front panel switch. Use the BIOS boot menu"
               echo "(usually F8) and choose IBA option (Intel Boot Agent)."
               # trash the MBR (for saftey this needs to uncommented)
               #/usr/bin/ssh $N dd if=/dev/zero of=/dev/sda bs=512 count=1
               #/usr/bin/ssh $N reboot
               sleep $SLEEPWAIT
            else
              DONE="yes"
            fi
          else
            echo "$N build successful."
            DONE="yes"
          fi
        else
          echo "Error: $NAME does not equal $N" 
          exit
        fi
      fi
    fi
    let COUNT=$COUNT+1
    if [ $COUNT -gt $MAXCOUNT ]; then
      echo "Loop timed out for $N aborting."
      DONE="yes"
    fi
  done
  let RELAYSTART=$RELAYSTART+1
  let IPSTART=$IPSTART+1
done

# turn off services and unmount files.
systemctl stop dhcp &> /dev/null
umount $HTTPISOPATH 1> /dev/null
sleep $SLEEPWAIT
pdsh uptime
echo "Build complete."

